import { IDisposable, ITextReader, EncodingType } from 'grain-sand-base';

declare const enum FileMimeTypes {
    TextPlain = "text/plain",
    TextUTF8 = "text/plain;charset=UTF-8",
    TextUTF16BE = "text/plain;charset=UTF-16BE",
    TextUTF16LE = "text/plain;charset=UTF-16LE",
    HTML = "text/html",
    CSS = "text/css",
    JavaScript = "text/javascript",
    XML = "text/xml",
    JSON = "application/json",
    CSV = "text/csv",
    Markdown = "text/markdown",
    LaTeX = "application/x-latex",
    TSV = "text/tab-separated-values",
    Bat = "text/x-sh",
    SimpleData = "text/x-simpledata",
    JPEG = "image/jpeg",
    PNG = "image/png",
    GIF = "image/gif",
    SVG = "image/svg+xml",
    WebP = "image/webp",
    BMP = "image/bmp",
    TIFF = "image/tiff",
    HEIF = "image/heif",
    HEIC = "image/heic",
    ICO = "image/x-icon",
    JPEG2000 = "image/jp2",
    AdobeXD = "application/vnd.adobe.xd",
    PSD = "image/vnd.adobe.photoshop",
    MP3 = "audio/mpeg",
    OGG = "audio/ogg",
    WAV = "audio/wav",
    AAC = "audio/aac",
    FLAC = "audio/flac",
    M4A = "audio/x-m4a",
    AMR = "audio/amr",
    WMA = "audio/x-ms-wma",
    MP4 = "video/mp4",
    MPEG = "video/mpeg",
    MP2T = "video/MP2T",
    MPEGPS = "video/MP1S",
    WebM = "video/webm",
    OggVideo = "video/ogg",
    QuickTime = "video/quicktime",
    AVI = "video/x-msvideo",
    MKV = "video/x-matroska",
    MOV = "video/quicktime",
    FLV = "video/x-flv",
    WMV = "video/x-ms-wmv",
    PDF = "application/pdf",
    EXE = "application/vnd.microsoft.portable-executable",
    MSI = "application/x-msdownload",
    APK = "application/vnd.android.package-archive",
    JAR = "application/java-archive",
    DMG = "application/x-apple-diskimage",
    ISO = "application/x-iso9660-image",
    TTF = "font/ttf",
    WOFF = "font/woff",
    WOFF2 = "font/woff2",
    OTF = "font/otf",
    EOT = "application/vnd.ms-fontobject",
    SVGFont = "font/svg+xml",
    DOC = "application/msword",
    DOCX = "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    XLS = "application/vnd.ms-excel",
    XLSX = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    PPT = "application/vnd.ms-powerpoint",
    PPTX = "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    ODT = "application/vnd.oasis.opendocument.text",
    ODS = "application/vnd.oasis.opendocument.spreadsheet",
    ODP = "application/vnd.oasis.opendocument.presentation",
    RTF = "application/rtf",
    Base64 = "application/base64",
    P7M = "application/pkcs7-mime",
    P12 = "application/x-pkcs12",
    COM = "application/x-msdos-program",
    DLL = "application/x-msdownload",
    SYS = "application/x-mswinexe",
    OLE = "application/x-oleobject",
    OctetStream = "application/octet-stream",
    UnknownBinary = "application/x-unknown",
    Binary = "application/x-binary",
    ZIP = "application/zip",
    RAR = "application/vnd.rar",
    GZIP = "application/gzip",
    TAR = "application/x-tar",
    BZIP2 = "application/x-bzip2",
    SEVEN_ZIP = "application/x-7z-compressed",
    XZ = "application/x-xz",
    GZ = "application/x-gzip",
    TAR_GZ = "application/gzip",
    LZ = "application/x-lzip",
    LZH = "application/x-lzh",
    Fbr = "application/zip+fbr",
    M3U = "audio/x-mpegurl",
    M3U8 = "application/x-mpegurl",
    PLS = "audio/x-scpls",
    XSPF = "application/xspf+xml",
    DPL = "application/x-daumplaylist"
}

declare const enum NetMimeTypes {
    MIME = "message/rfc822",
    SMTP = "application/smtp",
    HTTP = "application/http",
    HTTPS = "application/https",
    FTP = "application/ftp",
    TCP = "application/tcp",
    UDP = "application/udp",
    JSONLD = "application/ld+json",
    WebAssembly = "application/wasm",
    SVGZ = "image/svg+xml+gzip",
    MHTML = "multipart/related",
    OGG = "application/ogg",
    XAP = "application/x-silverlight-app",
    WOFF = "font/woff",
    WASM = "application/wasm",
    WGT = "application/widget",
    OctetStream = "application/octet-stream",
    UnknownBinary = "application/x-unknown"
}

declare const enum MimeTypes {
    SimpleData = "text/x-simpledata",
    Fbr = "application/zip+fbr",
    PSD = "image/vnd.adobe.photoshop",
    Bat = "text/x-sh",
    TextPlain = "text/plain",
    TextUTF8 = "text/plain;charset=UTF-8",
    TextUTF16LE = "text/plain;charset=UTF-16LE",
    TextUTF16BE = "text/plain;charset=UTF-16BE",
    HTML = "text/html",
    CSS = "text/css",
    JavaScript = "text/javascript",
    XML = "text/xml",
    JSON = "application/json",
    CSV = "text/csv",
    Markdown = "text/markdown",
    RTF = "application/rtf",
    LaTeX = "application/x-latex",
    TSV = "text/tab-separated-values",
    JPEG = "image/jpeg",
    PNG = "image/png",
    GIF = "image/gif",
    SVG = "image/svg+xml",
    WebP = "image/webp",
    BMP = "image/bmp",
    TIFF = "image/tiff",
    HEIF = "image/heif",
    HEIC = "image/heic",
    ICO = "image/x-icon",
    JPEG2000 = "image/jp2",
    MP3 = "audio/mpeg",
    OGG = "audio/ogg",
    WAV = "audio/wav",
    AAC = "audio/aac",
    FLAC = "audio/flac",
    M4A = "audio/x-m4a",
    AMR = "audio/amr",
    WMA = "audio/x-ms-wma",
    MP4 = "video/mp4",
    MPEG = "video/mpeg",
    MP2T = "video/MP2T",
    MPEGPS = "video/MP1S",
    WebM = "video/webm",
    OggVideo = "video/ogg",
    QuickTime = "video/quicktime",
    AVI = "video/x-msvideo",
    MKV = "video/x-matroska",
    MOV = "video/quicktime",
    FLV = "video/x-flv",
    WMV = "video/x-ms-wmv",
    PDF = "application/pdf",
    ZIP = "application/zip",
    RAR = "application/vnd.rar",
    SEVEN_ZIP = "application/x-7z-compressed",
    GZIP = "application/gzip",
    TAR = "application/x-tar",
    EXE = "application/vnd.microsoft.portable-executable",
    MSI = "application/x-msdownload",
    APK = "application/vnd.android.package-archive",
    JAR = "application/java-archive",
    DMG = "application/x-apple-diskimage",
    ISO = "application/x-iso9660-image",
    TTF = "font/ttf",
    WOFF = "font/woff",
    WOFF2 = "font/woff2",
    OTF = "font/otf",
    EOT = "application/vnd.ms-fontobject",
    SVGFont = "font/svg+xml",
    DOC = "application/msword",
    DOCX = "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    AdobeXD = "application/vnd.adobe.xd",
    XLS = "application/vnd.ms-excel",
    XLSX = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    PPT = "application/vnd.ms-powerpoint",
    PPTX = "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    ODT = "application/vnd.oasis.opendocument.text",
    ODS = "application/vnd.oasis.opendocument.spreadsheet",
    ODP = "application/vnd.oasis.opendocument.presentation",
    Base64 = "application/base64",
    P7M = "application/pkcs7-mime",
    P12 = "application/x-pkcs12",
    XZ = "application/x-xz",
    BZIP2 = "application/x-bzip2",
    GZ = "application/x-gzip",
    TAR_GZ = "application/gzip",
    LZ = "application/x-lzip",
    LZH = "application/x-lzh",
    COM = "application/x-msdos-program",
    DLL = "application/x-msdownload",
    SYS = "application/x-mswinexe",
    OLE = "application/x-oleobject",
    Binary = "application/x-binary",
    MIME = "message/rfc822",
    SMTP = "application/smtp",
    HTTP = "application/http",
    HTTPS = "application/https",
    FTP = "application/ftp",
    TCP = "application/tcp",
    UDP = "application/udp",
    JSONLD = "application/ld+json",
    WebAssembly = "application/wasm",
    SVGZ = "image/svg+xml+gzip",
    MHTML = "multipart/related",
    XAP = "application/x-silverlight-app",
    WASM = "application/wasm",
    WGT = "application/widget",
    OctetStream = "application/octet-stream",
    UnknownBinary = "application/x-unknown",
    M3U = "audio/x-mpegurl",
    M3U8 = "application/x-mpegurl",
    PLS = "audio/x-scpls",
    XSPF = "application/xspf+xml",
    DPL = "application/x-daumplaylist"
}

type FullMimeType = MimeTypes | `${MimeTypes};${string}`;
interface IExtensionMime {
    ext: string;
    mime: FullMimeType;
}

declare function getExtensionMime(extOrMine: string): IExtensionMime;

declare function parseMime(header: Uint8Array | ArrayBuffer | Blob): Promise<MimeTypes | undefined>;

declare function parseMimeExt(header: Uint8Array | ArrayBuffer | Blob): Promise<IExtensionMime>;

type Base64DataURL = `data:${string};base64,${string}`;
declare const decodeBase64: (input: string) => string;
declare const encodeBase64: (input: string) => string;

/**
 * 测试当前运行环境是否支持压缩
 */
declare function supportsCompression(): boolean;
/**
 * 压缩
 * @param data
 * @param format
 */
declare function compressData(data: ArrayBuffer | Uint8Array, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<ArrayBuffer>;
/**
 *  解压
 * @param compressedData
 * @param format
 */
declare function decompressData(compressedData: ArrayBuffer | Uint8Array, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<ArrayBuffer>;
/**
 * 压缩文本
 * @param data
 * @param format
 */
declare function compressText(data: string, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<ArrayBuffer>;
/**
 * 解压为文本
 * @param compressedData
 * @param format
 */
declare function decompressText(compressedData: ArrayBuffer | Uint8Array, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<string>;
/**
 * 压缩后转换为base64
 * @param data
 * @param format
 */
declare function compressToBase64(data: ArrayBuffer | Uint8Array, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<string>;
/**
 * 压缩文本为base64
 * @param data
 * @param format
 */
declare function compressTextToBase64(data: string, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<string>;
/**
 * 从base64中解压
 * @param base64
 * @param format
 */
declare function decompressFromBase64(base64: string, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<ArrayBuffer>;
/**
 * 从base64中解压为文本
 * @param base64
 * @param format
 */
declare function decompressTextFromBase64(base64: string, format?: 'gzip' | 'deflate' | 'deflate-raw'): Promise<string>;

type ValueCreator<T> = (key: string) => T;
declare class NamedWeakMap<T extends object = any> implements IDisposable {
    valueCreator?: ValueCreator<T>;
    private readonly objectRefs;
    constructor(valueCreator?: ValueCreator<T>);
    get(key: string): T | undefined;
    set(key: string, value: T): void;
    destroy(): void;
}

interface IExportMeta {
    readonly name: string;
    description?: string;
    extraData?: any;
}
interface IRecordMeta extends IExportMeta {
    fields: Record<string, number>;
    defaultValues?: Record<string, any>;
}

interface IExportData<T = any, K extends keyof T = keyof T | any> extends IExportMeta {
    fields?: Record<K, number>;
    defaultValues?: Record<K, any>;
    excludeFields?: K[];
    data: Iterable<T> | AsyncIterable<T>;
}
interface IDataExportArg {
    header: any;
    exportData: IExportData[];
}

declare const exportSimpleData: (arg: IDataExportArg) => Promise<string>;
declare class SimpleDataExport implements AsyncIterable<string> {
    private _arg?;
    constructor(arg: IDataExportArg);
    [Symbol.asyncIterator](): AsyncIterator<string>;
    destroy(): void;
}

type OnReadFn<T = any> = (name: string, page: T[], progress?: number, dataHeader?: IRecordMeta) => Promise<void> | void;
type OnNamedReadFn<T = any> = (page: T[], progress?: number, dataHeader?: IRecordMeta) => Promise<void> | void;
interface IReadRow<T> {
    name: string;
    data: T;
    progress: number;
    meta: IRecordMeta;
}
declare class SimpleDataReader {
    private _textReader?;
    constructor(dataOrReader: ITextReader | string);
    getHeader(): any;
    getSize(): number;
    read<T>(targetName: string): IterableIterator<IReadRow<T>>;
    read(): IterableIterator<IReadRow<any>>;
    getAll<T = any>(targetName: string): T[];
    pageRead(fn: OnReadFn, limit?: number): Promise<void>;
    namedRead(fnRecord: Record<string, OnNamedReadFn>, limit?: number): Promise<void>;
    destroy(): void;
}

declare function base64ToBlob(base64: Base64DataURL | string, mimeType: string): Blob;

declare const enum BlobTypes {
    /**
     * 文本类型
     */
    Text = 1,
    /**
     * 已编码的Base64URL
     */
    Base64 = 2,
    /**
     * <img>对象,类型为 HTMLImageElement
     */
    Image = 3,
    /**
     * 位图,类型为 ImageBitmap
     */
    Bitmap = 4,
    /**
     * Svg,类型为HTMLUnknownElement
     */
    Svg = 5,
    /**
     * Html,类型为Document
     */
    Html = 6,
    /**
     * Xml,类型为XMLDocument
     */
    Xml = 7,
    /**
     * Json,类型为any object
     */
    Json = 8,
    /**
     * 本类库内置的简单数据类型,比JSON有更小的空间占用,更利于网络传输
     * 类型为 SimpleDataReader
     */
    SimpleData = 9,
    /**
     * 当为不支持的类型时,返回原Blob与可能的类型
     */
    NotSupped = 10
}

type BlobResult = Blob | string | Base64DataURL | HTMLImageElement | ImageBitmap | HTMLUnknownElement | Document | XMLDocument;
interface IBlobPackage<T extends BlobResult = any> extends IExtensionMime {
    type: BlobTypes;
    blob: Blob;
    result?: T;
}

declare function readBlob(blob: Blob, encoding?: EncodingType): Promise<IBlobPackage>;
declare function readBlob(blob: Blob, type: BlobTypes.Base64): Promise<Base64DataURL>;
declare function readBlob(blob: Blob, type: BlobTypes.Image): Promise<HTMLImageElement>;
declare function readBlob(blob: Blob, type: BlobTypes.Bitmap): Promise<ImageBitmap>;
declare function readBlob(blob: Blob, type: BlobTypes.Svg): Promise<HTMLUnknownElement>;
declare function readBlob(blob: Blob, type: BlobTypes.Text, encoding?: EncodingType): Promise<string>;
declare function readBlob(blob: Blob, type: BlobTypes.Html, encoding?: EncodingType): Promise<Document>;
declare function readBlob(blob: Blob, type: BlobTypes.Xml, encoding?: EncodingType): Promise<XMLDocument>;
declare function readBlob(blob: Blob, type: BlobTypes.Json, encoding?: EncodingType): Promise<any>;
declare function readBlob(blob: Blob, type: BlobTypes.SimpleData, encoding?: EncodingType): Promise<SimpleDataReader>;
declare function readBlob(blob: Blob, type: BlobTypes, encoding: EncodingType): Promise<any>;

declare function readBlobData(blob: Blob, call: 'readAsText', encoding?: EncodingType): Promise<string>;
declare function readBlobData(blob: Blob, call: 'readAsDataURL'): Promise<string>;
declare function readBlobData(blob: Blob, call: 'readAsArrayBuffer'): Promise<ArrayBuffer>;

declare function miniToBlobType(miniType: MimeTypes): BlobTypes;

export { type Base64DataURL, type BlobResult, BlobTypes, FileMimeTypes, type FullMimeType, type IBlobPackage, type IDataExportArg, type IExportData, type IExportMeta, type IExtensionMime, type IReadRow, type IRecordMeta, MimeTypes, NamedWeakMap, NetMimeTypes, type OnNamedReadFn, type OnReadFn, SimpleDataExport, SimpleDataReader, type ValueCreator, base64ToBlob, compressData, compressText, compressTextToBase64, compressToBase64, decodeBase64, decompressData, decompressFromBase64, decompressText, decompressTextFromBase64, encodeBase64, exportSimpleData, getExtensionMime, miniToBlobType, parseMime, parseMimeExt, readBlob, readBlobData, supportsCompression };
